
/* $Revision: 1.5 $ */

/****************************************************************************
 *
 * PROJECT: X-Core / My Thesis
 *
 * MODULE:  Terrain Generators
 *
 * FILE:    VicarLabel.H
 *
 * DESCRIPTION: Read the label section of a vicar file into an STL Map
 *
 * EXPORTS:  
 *
 * NOTES:        
 *
 * (c) Copyright 2004 Kurt Schwehr. All rights reserved.
 ***************************************************************************/
// Include Guard
#ifndef __VICARLABEL_H_
#define __VICARLABEL_H_

#include <string>
#include <vector>
#include <map>
#include <algorithm>

// http://www.codeproject.com/vcpp/stl/PracticalGuideStl.asp
// The above explains how to make a class that works with Maps

// FIX: should Reals be double?
class VicLabel {
public:
  enum type {TYPE_UNKNOWN,INT,REAL,STRING,INTARRAY,REALARRAY,STRARRAY};

  VicLabel ();
  VicLabel (type t, std::string labelString);  // the whole string with =

  // Copy constructor
  VicLabel(const VicLabel &oldvc);
  void VicLabel::operator= (const VicLabel &oldvc);

  //
  // Generic labels
  //
  std::string getLabelName(); // return before the =
  std::string getLabelValStr(); // return after the =

  type getType() {return(labelType);}
  int getIntVal();
  float getRealVal();
  std::string getStringVal();
  std::string getWholeLabelStr(); // Return everything before AND after the '='
  void print(); // For debugging

private:
  std::string label;
  type labelType;
  int intVal;
  float realVal;
  std::string stringVal;
  std::vector<int> intArray;
  std::vector<float> realArray;
  std::vector<std::string> stringArray;
};

typedef std::map<std::string,VicLabel> VicLabelMap;


// Return a map of all the labels.
VicLabelMap LoadVicarLabelsFromFile(const std::string &filename);


#endif // __VICARLABEL_H_
