%{  /* -*- c++ -*-  */  // Emacs thinks this is a lisp file otherwise

/* $Revision: 1.9 $ */

/****************************************************************************
 *
 * PROJECT: X-Core / My Thesis
 *
 * MODULE:  Terrain Generators
 *
 * FILE:    VicarLabel.l
 *
 * DESCRIPTION: Flex parser of vicar labels.
 *
 * EXPORTS:  
 *
 * NOTES:        
 *
 *   - Only handles labels in the front block (FIX this)
 *   - Does not understand the meaning of the labels
 *   - FIX: support array types
 *   - Tested with dev version of flex 2.5.31, Mac OSX 10.3.3, gcc 3.3
 *   - Can only be run once per program run.
 *	FIX: reset the Labels map to empty on each run
 *   - NOT thread safe!!
 * 
 * SEE ALSO:
 * 
 *     VicarLabel.H
 *     XcVicarGrid.C which uses this parser
 *
 * (c) Copyright 2004 Kurt Schwehr. All rights reserved.
 ***************************************************************************/

  // STL tutorial including maps: http://www.yrl.co.uk/~phil/stl/stl.htmlx

#include <cassert>

#include <iostream>
#include "VicarLabel.H"

  extern "C" int vicwrap (void ){
    //std::cout << "vicwrap capped\n";
    // Do nothing
    return (1);
  }

  extern int viclex(); // Generated by flex>

  int yylex_vic (char *src,int len)
  {
    int result;
    YY_BUFFER_STATE buf_state;

    buf_state = yy_scan_bytes (src,len);
    result=viclex();
    yy_delete_buffer(buf_state);  // FIX: Is this really right?

    return result;  // 0 on success
  }


  VicLabel::VicLabel () {
    //std::cout << "default viclabel constructor\n";
    label="Unset";
    labelType = TYPE_UNKNOWN;
    intVal = -666;
    realVal = -66.6;
    stringVal = "no string";
  }

  // Copy constructor
  VicLabel::VicLabel(const VicLabel &oldvc) {
    //std::cout << "Copy constr\n";
    label = oldvc.label;
    labelType = oldvc.labelType;
    intVal = oldvc.intVal;
    realVal = oldvc.realVal;
    stringVal = oldvc.stringVal;
    intArray = oldvc.intArray;
    realArray = oldvc.realArray;
    stringArray = oldvc.stringArray;
  }

  // Overload = so it works in a map
  void VicLabel::operator= (const VicLabel &oldvc) {
    label = oldvc.label;
    labelType = oldvc.labelType;
    intVal = oldvc.intVal;
    realVal = oldvc.realVal;
    stringVal = oldvc.stringVal;
    intArray = oldvc.intArray;
    realArray = oldvc.realArray;
    stringArray = oldvc.stringArray;
  }

  // Return the label name before the '='
  std::string VicLabel::getLabelName() {
    size_t eq;
    eq=label.find("=");
    std::string s = label.substr(0,eq);
    return (s);
  }

  // return what is after the '=' in a label
  std::string VicLabel::getLabelValStr() {
    size_t eq;
    eq=label.find("=");
    std::string s = label.substr(eq+1,label.length());
    return (s);
  }

  int VicLabel::getIntVal() {
    if(INT!=labelType) {
      std::cerr << "bad access.  Not INT!\n";
      print();
      assert(false);
    }
    return(intVal);
  }
  float VicLabel::getRealVal() {
    if(REAL!=labelType) {
      std::cerr << "ERROR: bad access.  Not REAL!\n";
      print();
      assert(false);
    }
    return(realVal);
  }
  std::string VicLabel::getStringVal() {
    if(STRING!=labelType) {
      std::cerr << "ERROR: bad access.  Not STRING!\n";
      print();
      assert(false);
    }
    return(stringVal);
  }

  std::string VicLabel::getWholeLabelStr() {
    return(label);
  }

  // debugging
  void VicLabel::print() {
    std::cout << "\nLABEL: '" << label << "' ...\n";
    std::cout << " labelType: ";
    switch (labelType) {
    case INT:
      std::cout << "int\n";;
      break;
    case REAL:
      std::cout << "real\n";
      break;
    case STRING: 
      std::cout << "string\n";
     break;
    case INTARRAY:
      std::cout << "intarray\n";
      break;
    case REALARRAY:
      std::cout << "realarray\n";
      break;
    case STRARRAY:
      std::cout << "strarray\n";
      break;
    default:
      std::cout << "????\n";
      std::cerr << "ERROR: Unknown label type!!\n";
    }
    std::cout << "INT: " << intVal << "\n";
    std::cout << "REAL: " << realVal << "\n";
    std::cout << "STRING: " << stringVal << "\n";
  }

  // Fill in the data structure with a string "name=value" pair
  VicLabel::VicLabel (type t, std::string labelString) {
    label = labelString;
    labelType = t;

    labelType = TYPE_UNKNOWN;
    intVal = -666;
    realVal = -66.6;
    // Leave string empty

    switch (t) {
    case INT:
      labelType = INT;
      intVal = atoi(getLabelValStr().c_str());
      break;
    case REAL:
      labelType = REAL;
      realVal = atof(getLabelValStr().c_str());
      break;
    case STRING: 
      {
	labelType = STRING;
	std::string s = getLabelValStr();
	size_t start = s.find('\'');
	size_t end = s.rfind('\'');
	if (start>s.size() || end>s.size()) {
	  stringVal = "ERROR READING STRING\n";
	  std::cerr << "Error reading string: '"<<s<<"'\n";
	}
	stringVal = s.substr(start+1,end-1); // Drop the "" around the label value
	//stringVal = s.substr(1,s.length()-2); // Drop the '' around the label value
      }
     break;
    case INTARRAY:
      labelType = INTARRAY;
      std::cout << "FIX: handle INTARRAY label\n";
      assert(false);
      break;
    case REALARRAY:
      labelType = REALARRAY;
      std::cout << "FIX: handle REALARRAY label\n";
      assert(false);
      break;
    case STRARRAY:
      labelType = STRARRAY;
      std::cout << "FIX: handle STRARRAY  label\n";
      assert(false);
      break;
    default:
      std::cerr << "ERROR: Unknown label type!!\n";
      assert(false);
    }
    assert (TYPE_UNKNOWN != labelType);
  } // VicLabel::VicLabel


  // FIX: global variable that is not reset!
  //std::map <std::string,VicLabel> Labels;
  static VicLabelMap Labels;

%}

NUL		\0
WS		([ \t])
NL		\n
LETTER		([a-zA-Z])
DIGIT		([0-9])
SCALEFACTOR	(("e"|"E"|"D")("+"|"-")?{DIGIT}+)

/* Variables */
KEYLETTER	([A-Z_])
KEYWORD		({KEYLETTER}({KEYLETTER}|{DIGIT}){0,31})

STRLETTER	([^\'[:cntrl:]])

EQL		({WS}*"="{WS}*)
VICARLABEL	({KEYWORD}{EQL})

INTEGER		(("+"|"-")?{DIGIT}+)
REAL		(("+"|"-")?{DIGIT}+"."{DIGIT}*{SCALEFACTOR}?)
STRING		(\'{STRLETTER}*\')

LEFT		\(
RIGHT		\)
INTARRAY	({LEFT}({INTEGER},)+{INTEGER}{RIGHT})
REALARRAY	({LEFT}({REAL},)+{REAL}{RIGHT})
STRARRAY	({LEFT}({STRING},)+{STRING}{RIGHT})

%%

%{ /* Can place C code here */
%}

{VICARLABEL}{INTEGER}	{
  //std::cout << "INT: " << yytext << "\n";
  VicLabel l(VicLabel::INT,yytext);;
  Labels[l.getLabelName()] = l;
			}

{VICARLABEL}{REAL}	{
  //std::cout << "REAL: " << yytext << "\n";
  VicLabel l(VicLabel::REAL,yytext);;
  Labels[l.getLabelName()] = l;
			}

{VICARLABEL}{STRING}	{
  //std::cout << "STRING: " << yytext << "\n";
  VicLabel l(VicLabel::STRING,yytext);;
  Labels[l.getLabelName()] = l;
			}

{VICARLABEL}{INTARRAY}	{
  //std::cout << "INTARRAY: " << yytext << "\n";
  VicLabel l(VicLabel::INTARRAY,yytext);;
  Labels[l.getLabelName()] = l;
			}

{VICARLABEL}{REALARRAY}	{
  //std::cout << ": " << yytext << "\n";
  VicLabel l(VicLabel::REALARRAY,yytext);;
  Labels[l.getLabelName()] = l;
			}

{VICARLABEL}{STRARRAY}	{
  //std::cout << ": " << yytext << "\n";
  VicLabel l(VicLabel::STRARRAY,yytext);;
  Labels[l.getLabelName()] = l;
			}

{WS}*			{
}
{NL}* 			{
}
.			{
} 
<<EOF>>         	{ yyterminate(); }

%%  /* C code section */


#include <fstream>
using namespace std; // be lazy

VicLabelMap
LoadVicarLabelsFromFile(const std::string &filename) {
  Labels.clear();  // Nuke all entries currently in the global table

  std::ifstream in(filename.c_str(), ios::in|ios::binary|ios::ate);
  if (!in.is_open()) {
    std::cerr << "ERROR: Unable to open file '" << filename << "'\n";
  }

  //
  // READ Header which is LBLSIZE bytes long
  //
  in.seekg(8); // Skip LBLSIZE=
  int labelSize;
  in >> labelSize;
  if (in.fail()) {
    cerr << "ERROR: LBLSIZE read failed\n";
    exit (EXIT_FAILURE);
  } 
  char *buf = new char[labelSize+1];
  buf[labelSize] = '\0';  // Force null term string just in case

  in.seekg(0);
  in.read(buf,labelSize);
  if (in.fail()) {
    cerr << "ERROR: Read failed\n";
    exit (EXIT_FAILURE);
  }

  // Do the heavy lifting.  Results will be in Labels global variable
  const int r = yylex_vic(buf,labelSize);

  if (0!=r) {
    cerr << "ERROR: labels were not correctly read.\n"
	 << "  Continuing, but unpredictable results may follow.\n";
  }

  return (Labels);
}


//////////////////////////////////////////////////////////////////////
// TEST CODE
//////////////////////////////////////////////////////////////////////


#ifdef REGRESSION_TEST

int main (int argc, char *argv[]) {
  bool ok=true;

  if (true) {
    // FIX: make this an actual test
    VicLabel v(VicLabel::INT, "foo=31");
    std::string ls = v.getLabelName();
    std::string lv = v.getLabelValStr();
  }

  std::vector<std::string> files;
  files.push_back("TestData/gen4x4byteLinux.vic");
  files.push_back("TestData/gen4x4byteMac.vic");
  files.push_back("TestData/gen4x4fullLinux.vic");
  files.push_back("TestData/gen4x4fullMac.vic");
  files.push_back("TestData/gen4x4halfLinux.vic");
  files.push_back("TestData/gen4x4halfMac.vic");
  files.push_back("TestData/gen4x4realLinux.vic");
  files.push_back("TestData/gen4x4realMac.vic");

  size_t i=0;
  for (i=0;i < files.size();i++) {
    int r; // flex result code.  0 on success

    // ate opens at the end of the file
    ifstream in(files[i].c_str(), ios::in|ios::binary|ios::ate);
    const size_t fileSize=in.tellg();
    //cout << "FileSize = " << fileSize << "\n";

    //
    // READ Header which is LBLSIZE bytes long
    //
    in.seekg(8); // Skip LBLSIZE=
    int labelSize;
    in >> labelSize;
    if (in.fail()) {
      cerr << " LBLSIZE read failed\n";
      return (EXIT_FAILURE);
    } 

    if (fileSize<size_t(labelSize+1)) {
      std::cout << "  " << files[i] << ": FAILED.  (bad file size)";
      in.close();
      ok=false;
      continue;
    }

    char *buf = new char[labelSize+1];  // FIX:  need to delete this buffer!!
    buf[labelSize] = '\0';  // Force null term string just in case

    in.seekg(0);
    in.read(buf,labelSize);
    if (in.fail()) {
      cerr << "ERROR: Read failed\n";
      //return (EXIT_FAILURE); 
      ok=false;
      std::cout << "  " << files[i] << ": FAILED.\n";
      continue;
    }
    in.close();
    r = yylex_vic(buf,labelSize);
    std::cout << "  " << files[i] << ((0==r)?": ok.":": FAILED!") << std::endl;
    if (0!=r) ok=false;

    // FIX: add error check

    const int bands = Labels["NB"].getIntVal(); // Number of bands.
    // FIX: deal with more than one band.
    if (1<bands) {
      std::cerr << "WARNING: this file has more than one band. Not support yet\n";
      ok=false;
    }
    //const int startOfData = Labels["LBLSIZE"].getIntVal();

    std::string fmt=Labels["FORMAT"].getStringVal(); // BYTE, SHORT, REAL, etc
  } // for file

  std::cout << argv[0] << (ok?": ok.":": FAILED!")<<std::endl;
  return (ok?EXIT_SUCCESS:EXIT_FAILURE);
}

#endif // REGRESSION_TEST
