/* $Revision: 1.9 $ */

/****************************************************************************
 *
 * PROJECT: X-Core / My Thesis
 *
 * MODULE:  Terrain Generators
 *
 * FILE:    XcGridDrv.C
 *
 * DESCRIPTION: Driver interface to readers and writers
 *	Each driver inherits this interface.
 *
 * EXPORTS:  
 *
 * NOTES:        
 *
 *
 * (c) Copyright 2004 Kurt Schwehr. All rights reserved.
 ***************************************************************************/

// Include Guard
#ifndef __X_GRID_DRV_H_
#define __X_GRID_DRV_H_
/***************************************************************************
 * INCLUDES
 ***************************************************************************/

#include <string>
#include <vector>

#include "XcGridFile.H"

class XcGrid;

namespace XcG {
  // Read write options.  Drivers can choose to ingore these;
  class XcGridOpts {
  public:
    XcGridOpts();
    void print();

    // FIX: make accessors/settlers
    //private:
    //bool ascii; // true to write ascii, false to write binary. PGM
    int maxVal; // for pgm format 255 or 65xxx
    std::string textureFile;  // empty for no texture
    std::string cptFile;  // Color Palette Table
    // IV
    // colortable bool, color table object or file name?
    // normal spikes
    bool explicitNormals, colorTable, normalSpikes;

    // POVRAY
    bool includeCamera;  // povray

    // General things to do - see also terrain2-getopts.H
    //bool applyScaleOffset;
    //double zscale;
    //double zoffset;
  };


  // File type drivers base class
  class XcGridDrv {
  public:
    // FIX: why do we need an ID number?
    //virtual int getID() {return id;}
    //virtual void setID(int _id) {id=_id;}
    //int getID() {return id;}
    //void setID(int _id) {id=_id;}
    virtual std::string getName();
    virtual bool canReadType(const FileType t);
    virtual bool canWriteType(const FileType t);
    virtual std::vector<FileType> getReadTypes();
    virtual std::vector<FileType> getWriteTypes();
    // The ReadGrid does not do any of the generate functions for normals, triangles, etc.
    virtual bool ReadGrid(std::string filename, FileType t, XcGrid &grd);
    virtual bool WriteGrid(std::string filename, FileType t, XcGrid &grd, XcGridOpts &opts);
  private:
    int id;
  };

  typedef std::vector<XcGridDrv *> VecDriverPtr;

  VecDriverPtr XcGridDrvRegister();
  void XcGridDrvDelete(VecDriverPtr &drivers);
  size_t XcG::XcGridDrvGetReader (VecDriverPtr &drivers, XcG::FileType t);
  size_t XcG::XcGridDrvGetWriter (VecDriverPtr &drivers, XcG::FileType t);
  void XcGridDrvPrint(VecDriverPtr &drivers);
} // namespace XcG

#endif // __X_GRID_DRV_H_
