/* $Revision: 1.3 $ */

/****************************************************************************
 *
 * PROJECT: X-Core / My Thesis
 *
 * MODULE:  Terrain Generators
 *
 * FILE:    XcGridDrv.C
 *
 * DESCRIPTION: Driver interface to readers and writers
 *	This file registers all the known drivers.  It therefore depends on 
 *	all of them, making it no fun for the test code in each driver.  Each
 *	drivers test code has a register that ONLY registers itself.  Use this
 *	Register function for programs that want access to ALL of the drivers.
 *
 * EXPORTS:  
 *
 * NOTES:        
 *
 *
 * (c) Copyright 2004 Kurt Schwehr. All rights reserved.
 ***************************************************************************/


/***************************************************************************
 * INCLUDES
 ***************************************************************************/

#include "XcGridDrv.H" // Parent for all drivers

// Need an include here for each driver type.

#if HAVE_GMT
#include "XcGridGmt.H"	// Generic Mapping Tool (Gravity Mag Topo)
#endif
#if HAVE_GDAL
#include "XcGridGdal.H"	// Generic Mapping Tool (Gravity Mag Topo)
#endif
#include "XcGridIV.H"   // Open Inventor SoIndexedFaceSets
#include "XcGridPds.H"  // Planetary Data System
#include "XcGridPgm.H"	// Portable Gray Map (PNM tools / netpbm)
#include "XcGridPovRay.H"	// JPL VICAR planetary data format
#include "XcGridVicar.H"	// Point Of View Raytracer


//////////////////////////////////////////////////////////////////////
// BUILD/DESTROY DRIVER TABLE
//////////////////////////////////////////////////////////////////////

// FIX: use configure to select which drivers we can register
// This keeps the dependencies on XcGridDrv from being on every file type.
XcG::VecDriverPtr XcG::XcGridDrvRegister() {
  VecDriverPtr drivers;
#ifdef HAVE_GMT
  drivers.push_back(new XcGridDrvGmt);  // Generic Mapping Tool (Gravity Mag Topo)
#endif
#ifdef HAVE_GDAL
  drivers.push_back(new XcGridDrvGdal);  // http://remotesensing.org/gdal
#endif
  drivers.push_back(new XcGridDrvIv);   // Open Inventor SoIndexedFaceSets
  drivers.push_back(new XcGridDrvPds);  // Planetary Data System
  drivers.push_back(new XcGridDrvPgm);	// Portable Gray Map (PNM tools / netpbm)
  drivers.push_back(new XcGridDrvVicar);	// JPL VICAR planetary data format
  drivers.push_back(new XcGridDrvPovRay);	// Point Of View Raytracer

  return(drivers);
}

